class Agent:
    """Base Agent class for the ABM Framework."""

    def __init__(self, agent_id):
        """
        Initialize an Agent.

        Args:
            agent_id: Unique identifier for this agent
        """
        self.agent_id = agent_id

    def __repr__(self):
        return f"Agent({self.agent_id})"