import sys
from pathlib import Path

# Add framework src to path for imports
framework_src_path = Path(__file__).parent.parent.parent.parent / "src"
sys.path.insert(0, str(framework_src_path))

from layer1_core.managers.schedule import Schedule


class SchoolSchedule(Schedule):
    def __init__(self) -> None:
        super().__init__()